/*
 * support.c - Support routines for pppParse.
 *
 * Copyright (c) 1999 Kyler Jones.
 * All rights reserved. Please see the file LICENSE which should be
 * distributed with this software for terms of use.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "config.h"


extern	int total_access;
extern	int total_sbyte;
extern	int total_rbyte;
extern	int total_time, on_usr_cnt;
extern	char online_str[MAXPORTS][256], online_usr[MAXPORTS][24];
extern  int allcheck;

/*	The function get_line() retrieves a line from file fstream
	into char string buff.  I know there are C routines that could
	have done this for me, but I wanted something more flexible.
	I may have use in the future to alter how this works.
*/

int	get_line(FILE *fstream, char *buff)
{
	int i, eof_true = 0;
	i=-1;

	while(buff[i] != '\n' && !eof_true)
	{
		i++;
		buff[i]=fgetc(fstream);
		if(feof(fstream))
			eof_true = 1;
	}	
	buff[i+1]='\0';
	return eof_true;
}


/*	The function get_data() retrieves an ASCII string of numbers
	from char string buff and returns the integer equivalent.  Once
	again, I needed a more flexible routine than the standard CLIb
	routines.
*/ 

int	get_data(char *buff, char init_str[], int lnth, char term_char)
{
	char *tmp_str;
	char str[25], tmp_char;
	int i;

	tmp_str = strstr(buff, init_str) + lnth;

	i = 0;	

	while(tmp_str[i] != term_char)
	{
		str[i] = tmp_str[i]; 
		i++;		
	}	
	str[i] = '\0';
	return atoi(str);	
}


/*	The function get_name() retrieves the char string str from the
	char string buff, starting at the location of char string init_str, 
	+ lnth and ending at char term_char.
*/ 

void	get_name(char *buff,char init_str[],int lnth,char term_char,char str[])
{
	char *tmp_str;
	int i;

	tmp_str = strstr(buff, init_str) + lnth;
	
	i = 0;	

	while(tmp_str[i] != term_char)
	{
		str[i] = tmp_str[i]; 
		i++;		
	}	

	str[i] = '\0';

}


/*	The function online_chk() checks to see if a user us online.  This
	function is called if a successful login is detected and the final
	user statistics cannot be located.  The function checks to see if
	the login pppd PID is currently active, then looks in /var/run
	for the PID file used by pppd and checks to see if the time the
	process was started is the same as that in the syslog output.  This
	routine is not perfect since there is a couple of seconds between
	when pppd is started and when the PID file is created.  Therefore,
	this function only checks the Month, day and hour.
*/
 
int	online_chk(char *buff, int pid, char *port)
{
	char pid_entry[1000], month[10], spid[10], ps_cmd[256], ls_cmd[256];
	char ls_entry[1000], month_chk[10], time[10], time_chk[10], trash[256];
	FILE *psfile, *lsfile, *popen();
	int pid_chk, day, day_chk, found=0, entry_day;
	char hour[8];

	get_name(buff, "pppd", 5, ']', spid);
	sscanf(buff, "%s %d %s", month, &day, time); 

	strcpy(ps_cmd, PSCMD);
	strcat(ps_cmd, spid);

	if ((psfile = popen(ps_cmd, "r")) == NULL)
	{
		printf("Problem finding process, exiting...\n");
		exit(1);
	}
	
	if(get_line(psfile, pid_entry))
	{
		printf("Problem finding process, exiting...\n");
		exit(1);
	}

	while(!found)
	{
		if(get_line(psfile, pid_entry))
			break;
		sscanf(pid_entry, "%d", &pid_chk);
		if(pid_chk == pid && strstr(pid_entry, "pppd") != NULL)
		{
			strcpy(ls_cmd, LSCMD);
			strcat(ls_cmd, port);
			strcat(ls_cmd, ".pid");
		
			if ((lsfile = popen(ls_cmd,"r")) == NULL)
			{
				printf("Problem finding process, exiting...\n");
				exit(1);
			}
			get_line(lsfile, ls_entry);

			sscanf(ls_entry, "%s %s %s %s %s %s %d %s", trash
			, trash, trash, trash, trash,
			month_chk, &day_chk, time_chk);
			strncpy(hour, time_chk, 2);

/*	This is where the PID file's date is checked against the
	date in the log file.  (Right now I have disabled the hour check
	since I actually had users logging in 1 or 2 seconds before the
	end of an hour and the PID file was created after the hour.  This
	function will still fail if a user logs in 1 or 2 seconds before
	midnight you won't be up that late anyway ;^)  I wish someone
	would suggest a better way to check this.
*/

			if(strstr(month, month_chk) && day == day_chk)
			// && strstr(time, hour))
			{	
				found = 1;
				break;
			}
			pclose(lsfile); 
		}  
	}
	pclose(psfile);
	return found;
}


/*	This is where it all happens.  The function user_parse() pulls
	in information from file logfile, one line at a time, and collects
	data for USER user and MONTH month.  Keep in mind that this
	function looks from the beginning each time through in case the
	user logs in again while still online (in case you allow this).
	Please read the README file to see what key words are looked for
	in the log file for this to work correctly.
*/

int	user_parse(FILE *logfile, char user[],char month[], usrall *usr) 
{
	int i, bytecount, found, failure, eof_chk=0;
	int rbyte_count=0, sbyte_count=0, access_count=0, online_count=0;
	char login_entry[2500], connect_str[2500], port[25], tmp_str[24];
	char entry[2500], tmp_entry[2500], log, entry_date[25], user_name[25];
	int pid;
	int online, fail_cnt=0;	
	fpos_t fposa, fposb;

/*	Main loop - go through the file line by line until end-of-file
	is reached and look for all log in's by user.
*/
	
	while(!eof_chk)
	{

	if(get_line(logfile, entry))
		break;	

/*
	We need the line that displays the connection info for later.
*/

	if(strstr(entry, "Connect:"))
		strcpy(connect_str, entry);



/*	Check for user logging in.  This could be tweaked depending
	on your syslog output.  See the README file for details.
*/

	if(strstr(entry, "pppd") != NULL && (strstr(entry, "logged in")
	!= NULL || strstr(entry, "started by") != NULL)
	&& strstr(entry, user) != NULL && ((strstr(entry,month) != NULL)
	|| (strstr(month, "all") != NULL)))

	{


/*	I had to add this in case one user's username is a substring
	of another user's username.
*/
		if(strstr(entry, "started by") == NULL)
			get_name(entry, "user", 5, ' ', user_name);
		else
			get_name(entry, "by", 3, ',', user_name);
		if(strcmp(user, user_name) != 0)	
			continue;

/*	We have a login by the correct user, start reporting.
*/
		access_count += 1;		
		strncpy(entry_date, entry, 15);
		strcpy(login_entry, entry);	
		strcpy(usr->info[access_count], entry_date);

/*	Save location in file so that we can go back after looking
	for current login statistics.  This allows us to check if the
	same user logged in twice.  Also, check for end-of-file while
	looking for the next line. 
*/

		fgetpos(logfile, &fposa);
		if(get_line(logfile, tmp_entry))
		{
			printf("End of file reached\n");	
			exit(1);
		}
		fgetpos(logfile, &fposb);
		fsetpos(logfile, &fposa);

/*	Get the PID of the current user log in.
*/
		pid = get_data(entry, "pppd", 5, ']');
		found = 0;
		failure = 0;

/*	Go through the file one line at a time and check for connection
	time statistics for the above PID.
*/
		
		while(!found)
		{	

/*	If end-of-file is reached before the user statistics are
	found, either the user is still logged in, or something bad
	happened to pppd (or your system) before the statistics
	were generated.
*/

			if(get_line(logfile, entry))
			{
			   get_name(connect_str, "Connect", 9, ' ', port);
			   if(online_chk(login_entry, pid, port))
			   {
				strcat(usr->info[access_count],"\t    Online");	
				strcpy(online_str[on_usr_cnt], connect_str);
				strcpy(online_usr[on_usr_cnt], user_name);	
				on_usr_cnt += 1;	
				break;
			   }
			   else
			   {
				strcat(usr->info[access_count], "\t    Unknown");	
			   }
			   break;
			}

/*	Check if the login failed (there may be more types of failure
	key words than this routine checks for.  So, once again, this
	might be a good condition to tweak.
*/
			if(strstr(entry, "pppd") != NULL && (strstr(entry,
			"login failure") != NULL || strstr(entry, "failed")
			!= NULL) && get_data(entry, "pppd",
			5, ']') == pid)
			{
				strcat(usr->info[access_count], 
						"\t    Login Failure");
				fail_cnt += 1;
				failure = 1;
				break;
			}	

/*	Once again, see the README file for what key words are looked
	for.  You could tweak these if necessary.  If key words are
	found, check against the PID above to ensure the user hasn't
	logged in more than once.
*/
			if(strstr(entry, "pppd") != NULL && strstr(entry, 
			"Connect time") != NULL)
			{
				if (get_data(entry, "pppd", 5, ']') == pid)	
				{
					found = 1;	
					online = get_data(entry,"time", 5,' ');
					online_count += online;
					get_name(entry, "time", 5, '.',tmp_str);
					strcat(usr->info[access_count], "\t    ");
					strcat(usr->info[access_count],tmp_str);
				}
			}
		}

/*	Reset the file and check for data sent/received statistics.
*/

		fsetpos(logfile, &fposa);	
		found = 0;

		while(!found && !failure)
		{	

	   	   if(get_line(logfile, entry))
		   {

			get_name(connect_str, "Connect", 9, ' ', port);
			if(online_chk(login_entry, pid, port))
			{
				strcat(usr->info[access_count], 
						"\tOnline\t\tOnline");	
				break;
			}
			else
			{	
				strcat(usr->info[access_count], 
						"\tUnknown\t\tUnknown");	
				break;
			}
		   }

		   if(strstr(entry, "pppd") != NULL && strstr(entry, 
		   "Sent") != NULL)
		   {
			if (get_data(entry, "pppd", 5, ']') == pid)	
			{
			   found = 1;
			   get_name(entry,"received", 9, ' ', tmp_str);	
			   strcat(usr->info[access_count], "\t\t");
			   strcat(usr->info[access_count], tmp_str);
			   sbyte_count += get_data(entry, "received", 9, ' ');
			   get_name(entry,"Sent", 5, ' ', tmp_str);	
			   strcat(usr->info[access_count], "\t\t");
			   strcat(usr->info[access_count], tmp_str);
			   rbyte_count += get_data(entry, "Sent", 5, ' ');
			}
		   }
		}
		
/* 	Reset the file, go back and look for the next log in by user.
*/

	fsetpos(logfile, &fposb);	

	}
	}


/*	Add statistics for user to global statistics.
*/
 	if(!allcheck)
	{
		total_access += (access_count - fail_cnt);
		total_sbyte += sbyte_count;
		total_rbyte += rbyte_count;
		total_time += online_count;
	}	

	usr->access_count = (access_count - fail_cnt);
	usr->sbyte_count = sbyte_count;
	usr->rbyte_count = rbyte_count;
	usr->online_count = online_count;

	return(access_count);
	
}

